@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2014 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2015 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -71,6 +71,10 @@ public:
 	}
 
 private:
+	// Undefined copy constructor and assignment operator
+	CMutex(const CMutex&);
+	CMutex& operator=(const CMutex&);
+
 	pthread_mutex_t m_mutex;
 };
 
@@ -105,6 +109,10 @@ public:
 	}
 
 private:
+	// Undefined copy constructor and assignment operator
+	CMutexLocker(const CMutexLocker&);
+	CMutexLocker& operator=(const CMutexLocker&);
+
 	CMutex &m_mutex;
 	bool m_locked;
 };
@@ -161,6 +169,10 @@ public:
 	}
 
 private:
+	// Undefined copy constructor and assignment operator
+	CConditionVariable(const CConditionVariable&);
+	CConditionVariable& operator=(const CConditionVariable&);
+
 	pthread_cond_t m_cond;
 };
 
@@ -185,25 +197,85 @@ public:
 			exit(1);
 		}
 	}
+
+private:
+	// Undefined constructor
+	CThread();
 };
 
+/**
+ * A job is a task which should run without blocking the main thread. You do
+ * this by inheriting from this class and implementing the pure virtual methods
+ * runThread(), which gets executed in a separate thread and does not block the
+ * main thread, and runMain() which gets automatically called from the main
+ * thread after runThread() finishes.
+ *
+ * After you create a new instance of your class, you can pass it to
+ * CThreadPool()::Get().addJob(job) to start it. The thread pool automatically
+ * deletes your class after it finished.
+ *
+ * For modules you should use CModuleJob instead.
+ */
 class CJob {
 public:
+	friend class CThreadPool;
+
+	enum EJobState {
+		READY,
+		RUNNING,
+		DONE,
+		CANCELLED
+	};
+
+	CJob() : m_eState(READY) {}
+
+	/// Destructor, always called from the main thread.
 	virtual ~CJob() {}
+
+	/// This function is called in a separate thread and can do heavy, blocking work.
 	virtual void runThread() = 0;
+
+	/// This function is called from the main thread after runThread()
+	/// finishes. It can be used to handle the results from runThread()
+	/// without needing synchronization primitives.
 	virtual void runMain() = 0;
+
+	/// This can be used to check if the job was cancelled. For example,
+	/// runThread() can return early if this returns true.
+	bool wasCancelled() const;
+
+private:
+	// Undefined copy constructor and assignment operator
+	CJob(const CJob&);
+	CJob& operator=(const CJob&);
+
+	// Synchronized via the thread pool's mutex! Do not access without that mutex!
+	EJobState m_eState;
 };
 
 class CThreadPool {
 private:
+	friend class CJob;
+
 	CThreadPool();
 	~CThreadPool();
 
 public:
 	static CThreadPool& Get();
 
+	/// Add a job to the thread pool and run it. The job will be deleted when done.
 	void addJob(CJob *job);
 
+	/// Cancel a job that was previously passed to addJob(). This *might*
+	/// mean that runThread() and/or runMain() will not be called on the job.
+	/// This function BLOCKS until the job finishes!
+	void cancelJob(CJob *job);
+
+	/// Cancel some jobs that were previously passed to addJob(). This *might*
+	/// mean that runThread() and/or runMain() will not be called on some of
+	/// the jobs. This function BLOCKS until all jobs finish!
+	void cancelJobs(const std::set<CJob *> &jobs);
+
 	int getReadFD() const {
 		return m_iJobPipe[0];
 	}
@@ -211,11 +283,14 @@ public:
 	void handlePipeReadable() const;
 
 private:
-	void jobDone(const CJob* pJob) const;
+	void jobDone(CJob* pJob);
 
 	// Check if the calling thread is still needed, must be called with m_mutex held
 	bool threadNeeded() const;
 
+	CJob *getJobFromPipe() const;
+	void finishJob(CJob *) const;
+
 	void threadFunc();
 	static void *threadPoolFunc(void *arg) {
 		CThreadPool &pool = *reinterpret_cast<CThreadPool *>(arg);
@@ -229,6 +304,12 @@ private:
 	// condition variable for waiting idle threads
 	CConditionVariable m_cond;
 
+	// condition variable for reporting finished cancellation
+	CConditionVariable m_cancellationCond;
+
+	// condition variable for waiting running threads == 0
+	CConditionVariable m_exit_cond;
+
 	// when this is true, all threads should exit
 	bool m_done;
 
