@@ -1454,7 +1454,7 @@ public:
 
 		CUser* pNewUser = GetNewUser(WebSock, pUser);
 		if (!pNewUser) {
-			WebSock.PrintErrorPage("Invalid user settings");
+			// GetNewUser already called WebSock.PrintErrorPage()
 			return true;
 		}
 
