@@ -130,7 +130,7 @@ void CChan::Cycle() const {
 }
 
 void CChan::JoinUser(const CString& sKey) {
-	if (!sKey.empty()) {
+	if (!IsOn() && !sKey.empty()) {
 		SetKey(sKey);
 	}
 	m_pNetwork->PutIRC("JOIN " + GetName() + " " + GetKey());
