@@ -161,7 +161,9 @@ void CWebAuth::AcceptedLogin(CUser& User) {
 
 		m_pWebSock->SetLoggedIn(true);
 		m_pWebSock->UnPauseRead();
-		if (!m_bBasic) {
+		if (m_bBasic) {
+			m_pWebSock->ReadLine("");
+		} else {
 			m_pWebSock->Redirect("/?cookie_check=true");
 		}
 
@@ -178,7 +180,13 @@ void CWebAuth::RefusedLogin(const CString& sReason) {
 
 		m_pWebSock->SetLoggedIn(false);
 		m_pWebSock->UnPauseRead();
-		m_pWebSock->Redirect("/?cookie_check=true");
+		if (m_bBasic) {
+			m_pWebSock->AddHeader("WWW-Authenticate", "Basic realm=\"ZNC\"");
+			m_pWebSock->CHTTPSock::PrintErrorPage(401, "Unauthorized", "HTTP Basic authentication attemped with invalid credentials");
+			// Why CWebSock makes this function protected?..
+		} else {
+			m_pWebSock->Redirect("/?cookie_check=true");
+		}
 
 		DEBUG("UNSUCCESSFUL login attempt ==> REASON [" + sReason + "] ==> SESSION [" + spSession->GetId() + "]");
 	}
@@ -482,13 +490,19 @@ CWebSock::EPageReqResult CWebSock::PrintTemplate(const CString& sPageName, CStri
 	m_Template["PageName"] = sPageName;
 
 	if (pModule) {
-		CUser* pUser = pModule->GetUser();
-		m_Template["ModUser"] = pUser ? pUser->GetUserName() : "";
 		m_Template["ModName"] = pModule->GetModName();
 
 		if (m_Template.find("Title") == m_Template.end()) {
 			m_Template["Title"] = pModule->GetWebMenuTitle();
 		}
+
+		std::vector<CTemplate*>* breadcrumbs = m_Template.GetLoop("BreadCrumbs");
+		if (breadcrumbs->size() == 1 && m_Template["Title"] != pModule->GetModName()) {
+			// Module didn't add its own breadcrumbs, so add a generic one...
+			// But it'll be useless if it's the same as module name
+			CTemplate& bread = m_Template.AddRow("BreadCrumbs");
+			bread["Text"] = m_Template["Title"];
+		}
 	}
 
 	if (!m_bPathsSet) {
@@ -777,6 +791,10 @@ CWebSock::EPageReqResult CWebSock::OnPageRequestInternal(const CString& sURI, CS
 		} else {
 			SetPaths(pModule, true);
 
+			CTemplate& breadModule = m_Template.AddRow("BreadCrumbs");
+			breadModule["Text"] = pModule->GetModName();
+			breadModule["URL"] = pModule->GetWebPath();
+
 			/* if a module returns false from OnWebRequest, it does not
 			   want the template to be printed, usually because it did a redirect. */
 			if (pModule->OnWebRequest(*this, m_sPage, m_Template)) {
